/// <reference types="react-native" />
import { JSONCandidate } from '@mj-studio/js-util';
export declare type RestMethod = 'GET' | 'POST' | 'PUT' | 'PATCH' | 'DELETE' | 'OPTIONS' | 'HEAD';
export declare type Header = {
    [P in string]: string;
} & {
    'Content-Type'?: ContentType;
    Accept?: ContentType;
    Authorization?: string;
};
export declare type ReactNativeFile = {
    key: string;
    file: {
        name: string;
        uri: string;
        type: string;
    };
};
export declare type ContentType = 'application/json' | 'application/x-www-form-urlencoded;charset=UTF-8' | 'multipart/form-data';
export declare type RequestOptions<ResponseData> = {
    queryParams?: object;
    body?: object | URLSearchParams;
    files?: ReactNativeFile[];
    headers?: Header;
    interceptor?: (json: any) => ResponseData;
    mock?: ResponseData;
    mockError?: any;
    enableMock?: boolean;
    baseUrl?: string;
    meta?: any;
    useRawUrl?: boolean;
    credentials?: RequestCredentials_;
};
export declare type Unsubscribe = () => void;
export declare type ApiResult<ResponseData = any> = Promise<ResponseData>;
export declare type RequestOptionsInterceptor<ResponseData extends JSONCandidate> = (request: RequestOptions<ResponseData>, meta: {
    url: string;
    method: RestMethod;
    baseUrl: string;
}) => Promise<RequestOptions<ResponseData>>;
export declare type ResponseDataInterceptor<ResponseData extends JSONCandidate> = (responseData: ResponseData, statusCode: number, url: string, method: RestMethod, meta?: any) => Promise<ResponseData>;
export declare type ErrorInterceptorParams = {
    error: any;
    statusCode?: number;
    url?: string;
    body?: any;
    queryParams?: any;
};
export declare type ErrorInterceptor = (params: ErrorInterceptorParams) => any;
export declare type Settings<ResponseData extends JSONCandidate> = {
    headers: Header;
    baseUrl: string;
    errorInterceptor: ErrorInterceptor;
    requestInterceptor: RequestOptionsInterceptor<ResponseData>;
    responseInterceptor: ResponseDataInterceptor<ResponseData>;
    responseCodeWhiteListRange: {
        minInclude: number;
        maxExclude: number;
    };
    responseCodeWhiteList: number[];
    responseCodeBlackList: number[];
    serializedNames: Record<string, string>;
    credentials?: RequestCredentials_;
};
declare let settings: Settings<any>;
export declare function setApiDefaultSettings(options: Partial<typeof settings>): void;
export declare function clearApiDefaultSettings(): void;
export declare function getApiDefaultSettings(): Partial<typeof settings>;
declare function request<ResponseData = unknown>(method: RestMethod, url: string, options?: RequestOptions<ResponseData>): ApiResult<ResponseData>;
export default request;
